/* Copyright (C) 2022 GSI Helmholtzzentrum fuer Schwerionenforschung, Darmstadt
   SPDX-License-Identifier: GPL-3.0-only
   Authors: Sergey Gorbunov, Sergei Zharko [committer] */

#include "L1Station.h"

#include <Logger.h>

#include <iomanip>
#include <sstream>

//------------------------------------------------------------------------------------------------------------------------------------
//
void L1Station::CheckConsistency() const
{
  /*
   * Integer fields initialization checks
   */

  if (type < 0) {
    std::stringstream msg;
    msg << "L1Station: station type was not initialized (type = " << type << ", type > 0)";
    throw std::logic_error(msg.str());
  }

  if (timeInfo != 0 && timeInfo != 1) {
    std::stringstream msg;
    msg << "L1Station: illegal time information flag (timeInfo = " << timeInfo << ", "
        << "0 [time information is not used] or 1 [time information is used] expected)";
    throw std::logic_error(msg.str());
  }

  if (fieldStatus != 0 && fieldStatus != 1) {
    std::stringstream msg;
    msg << "L1Station: illegal field status flag (fieldStatus = " << fieldStatus << ", "
        << "0 [station is outside the field] or 1 [station is inside the field] expected";
    throw std::logic_error(msg.str());
  }

  /*
   * SIMD vector checks: all the words in a SIMD vector must be equal
   */

  L1Utils::CheckSimdVectorEquality(fZ, "L1Station::fZ");
  L1Utils::CheckSimdVectorEquality(Xmax, "L1Station::Xmax");
  L1Utils::CheckSimdVectorEquality(Ymax, "L1Station::Ymax");

  /*
   * Inner and outer radia checks:
   *  (i)  both Xmax and Ymax must be > 0
   */

  if (Xmax[0] < 0. || Ymax[0] < 0.) {
    std::stringstream msg;
    msg << "L1Station: " << this->ToString() << " has incorrect sizes: "
        << "Xmax = " << Xmax[0] << " [cm], Ymax = " << Ymax[0] << " [cm] (0 < Xmax && 0 < Ymax expected)";
    throw std::logic_error(msg.str());
  }


  /*
   * Check consistency of other members
   */

  //materialInfo.CheckConsistency();
  // TODO: Temporarily switched off, because Much has RL = 0, which is actually incorrect, but really is not used.
  //       One should provide average radiation length values for each Much layer (S.Zharko)
  fieldSlice.CheckConsistency();
}

// TODO: Improve log style (S.Zh.)
//------------------------------------------------------------------------------------------------------------------------------------
//
void L1Station::Print(int verbosity) const { LOG(info) << ToString(verbosity); }

//------------------------------------------------------------------------------------------------------------------------------------
//
std::string L1Station::ToString(int verbosityLevel, int indentLevel) const
{
  std::stringstream aStream {};
  constexpr char indentChar = '\t';
  std::string indent(indentLevel, indentChar);
  if (verbosityLevel == 0) { aStream << "station type = " << type << ", z = " << fZ[0] << " cm"; }
  else {
    aStream << '\n';
    aStream << indent << "z pos [cm]:   " << std::setw(12) << std::setfill(' ') << fZ[0] << '\n';
    aStream << indent << "X_max [cm]:   " << std::setw(12) << std::setfill(' ') << Xmax[0] << '\n';
    aStream << indent << "Y_max [cm]:   " << std::setw(12) << std::setfill(' ') << Ymax[0] << '\n';
    aStream << indent << "Station type: " << std::setw(12) << std::setfill(' ') << type << '\n';
    aStream << indent << "Is time used: " << std::setw(12) << std::setfill(' ') << timeInfo << '\n';
    aStream << indent << "Is in field:  " << std::setw(12) << std::setfill(' ') << fieldStatus << '\n';

    if (verbosityLevel > 3) {
      aStream << indent << "Field approcimation coefficients:\n";
      aStream << fieldSlice.ToString(indentLevel + 1) << '\n';
    }
  }
  return aStream.str();
}
